package operation

import (
	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/web"
	kit "shylinux.com/x/toolkits"

	"2025-dev.shylinux.com/x/20250215-cluster/src/travel"
	"shylinux.com/x/operation/src/operation/model"
)

type Cluster struct {
	Table
	travel travel.Travel
	order  string `data:"1"`
	fields string `data:"title,content,space,total,stock,price,cluster_status"`
	payfor string `name:"payfor" help:"购买" style:"notice" role:"leader"`
	modify string `name:"modify title content price" style:"danger"`
}

func (s Cluster) Scan(m *ice.Message, arg ...string) {
	m.Cmd(s.travel, s.travel.Concurrent, ice.GetTypeMod(s.travel), ice.GetTypeKey(s.travel)).Table(func(value ice.Maps) {
		s.InsertIfNeed(m, web.SPACE, value[web.SPACE])
	})
	m.Cmd("").Table(func(value ice.Maps) { s.ScanService(m.Options(value)) })
	m.ProcessRefresh()
}
func (s Cluster) List(m *ice.Message, arg ...string) {
	if len(arg) < 2 {
		if m.IsTech() {
			s.Select(m)
		} else {
			s.Select(m, model.STATUS, kit.Format(ClusterOnline))
		}
	} else {
		s.SelectDetail(m, model.UID, arg[1])
	}
	if m.Length() == 0 {
		if m.IsTech() {
			m.EchoInfoButton("请扫描集群", s.Scan)
		} else {
			m.Echo("请联系「平台管理员」部署集群").Action()
		}
	} else {
		if m.Display(""); m.IsTech() {
			m.Table(func(value ice.Maps) {
				if ClusterStatus(kit.Int(value[model.CLUSTER_STATUS])) == ClusterOnline && kit.Int(value[model.STOCK]) > 0 {
					m.PushButton(s.Payfor, s.ScanService, s.Offline, s.Modify)
				} else {
					m.PushButton(s.ScanService, s.Online, s.Modify)
				}
			}).Action(s.Scan)
		} else if s.IsLeader(m) {
			m.Table(func(value ice.Maps) {
				if ClusterStatus(kit.Int(value[model.CLUSTER_STATUS])) == ClusterOnline && kit.Int(value[model.STOCK]) > 0 {
					m.PushButton(s.Payfor)
				} else {
					m.PushButton()
				}
			}).Action()
		} else {
			m.PushAction().Action()
		}
	}
	if len(arg) == 2 && m.IsTech() {
		m.EchoIFrame(web.S(m.Append(web.SPACE)))
	}
}
func (s Cluster) Payfor(m *ice.Message, arg ...string) {
	m.Cmdy(Gateway{}, s.Payfor, arg, kit.Dict(model.CLUSTER_UID, m.Option(model.UID)))
}
func (s Cluster) ScanService(m *ice.Message, arg ...string) {
	m.Cmdy(Gateway{}, s.Scan, arg, kit.Dict(model.CLUSTER_UID, m.Option(model.UID))).ProcessRefresh()
	s.Update(m, m.AppendSimple(model.TOTAL, model.STOCK), m.OptionSimple(model.UID)...)
}
func (s Cluster) Offline(m *ice.Message, arg ...string) {
	s.Update(m, []string{model.STATUS, kit.Format(ClusterOffline)})
	s.RecordEventWithName(m, "")
}
func (s Cluster) Online(m *ice.Message, arg ...string) {
	s.Update(m, []string{model.STATUS, kit.Format(ClusterOnline)})
	s.RecordEventWithName(m, "")
}
func (s Cluster) Modify(m *ice.Message, arg ...string) {
	s.Update(m, s.TransPrice(m, arg))
	s.RecordEventWithName(m, "")
}

func init() { ice.TeamCtxCmd(Cluster{}) }

type ClusterStatus int

const (
	ClusterOffline ClusterStatus = iota
	ClusterOnline
)

var ClusterStatusList = map[ClusterStatus]string{
	ClusterOffline: "offline",
	ClusterOnline:  "online",
}

func (s ClusterStatus) String() string { return ClusterStatusList[s] }
